package com.yiidata.mvc.handle;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Sets;
import com.yiidata.mvc.annaotion.Path;
import freemarker.template.TemplateException;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.lang.reflect.Method;
import java.util.*;

/**
 * <pre>
 *
 * Created by zhenqin.
 * User: zhenqin
 * Date: 17/12/13
 * Time: 17:51
 * Vendor: yiidata.com
 * To change this template use File | Settings | File Templates.
 *
 * </pre>
 *
 * @author zhenqin
 */
public class DynamicRequestServlet extends HttpServlet {


    /**
     * Action Object
     */
    private final Object instance;


    /**
     * Action Method
     */
    private final Method method;


    /**
     * 当前方法上 Path
     */
    protected final Path pathAnno;


    /**
     * LOG
     */
    protected static Logger LOG = LoggerFactory.getLogger(DynamicRequestServlet.class);


    public DynamicRequestServlet(Object instance, Method method, Path pathAnno) {
        this.instance = instance;
        this.method = method;
        this.pathAnno = pathAnno;
    }


    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        Set<RequestMethod> methods = Sets.newHashSet(pathAnno.method());
        if(methods.contains(RequestMethod.GET)) {
            doAction(req, resp);
        } else {
            LOG.warn("path {} Request method 'GET' not supported", req.getRequestURI());
            unSupportAction(req, resp, "GET");
        }
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        Set<RequestMethod> methods = Sets.newHashSet(pathAnno.method());
        if(methods.contains(RequestMethod.POST)) {
            doAction(req, resp);
        } else {
            LOG.warn("path {} Request method 'POST' not supported", req.getRequestURI());
            unSupportAction(req, resp, "POST");
        }
    }


    public void unSupportAction(HttpServletRequest req, HttpServletResponse resp, String method) throws ServletException, IOException{
        HashMap<String, Object> params = new HashMap<>();
        params.put("request", req);
        params.put("date", new Date());
        params.put("method", method);

        ServletOutputStream out = resp.getOutputStream();
        try {
            resp.setCharacterEncoding("utf-8");
            resp.setStatus(405);
            resp.setContentType("text/html");
            resp.setHeader("Content-Type", "text/html; charset=utf-8");
            new FreemarkerView("unsupport_action.ftl").out(new OutputStreamWriter(out, "utf-8"), params);
        } catch (TemplateException e) {
            resp.reset();
            resp.setCharacterEncoding("utf-8");
            resp.setStatus(503);
            resp.setContentType("text/html; charset=utf-8");
            resp.setHeader("Content-Type", "text/html; charset=utf-8");
            out.write(ExceptionUtils.getFullStackTrace(e).getBytes("utf-8"));
            throw new ServletException(e);
        } finally {
            out.flush();
        }
    }


    /**
     * 执行调用
     * @param req 请求
     * @param resp 相应
     */
    public void doAction(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException{
        HashMap<String, Object> params = new HashMap<>();
        params.put("request", req);

        ServletOutputStream out = resp.getOutputStream();
        try {
            Object r = method.invoke(instance, params, req, resp);
            if(r != null) {
                if(r instanceof String) {
                    resp.setCharacterEncoding(pathAnno.charset());
                    resp.setContentType(pathAnno.contentType() + "; charset=" + pathAnno.charset());
                    resp.setHeader("Content-Type", pathAnno.contentType()+ "; charset=" + pathAnno.charset());
                    new FreemarkerView((String)r).out(new OutputStreamWriter(out, pathAnno.charset()), params);
                } else if((r instanceof Map) || (r instanceof Collection)) {
                    resp.setCharacterEncoding(pathAnno.charset());
                    resp.setContentType(pathAnno.contentType() + "; charset=" + pathAnno.charset());
                    resp.setHeader("Content-Type", pathAnno.contentType()+ "; charset=" + pathAnno.charset());
                    out.write(JSON.toJSONString(r).getBytes(pathAnno.charset()));
                } else {
                    resp.setCharacterEncoding(pathAnno.charset());
                    resp.setContentType(pathAnno.contentType() + "; charset=" + pathAnno.charset());
                    resp.setHeader("Content-Type", pathAnno.contentType()+ "; charset=" + pathAnno.charset());
                    out.write(JSON.toJSONString(r).getBytes(pathAnno.charset()));
                }
            }
        } catch (Exception e) {
            resp.reset();
            resp.setCharacterEncoding("utf-8");
            resp.setStatus(503);
            resp.setContentType("text/html; charset=utf-8");
            resp.setHeader("Content-Type", "text/html; charset=utf-8");
            out.write(ExceptionUtils.getFullStackTrace(e).getBytes("utf-8"));
            throw new ServletException(e);
        } finally {
            out.flush();
        }
    }

}
